@@ -1,5 +1,5 @@
 from dataclasses import dataclass, field
-from typing import List, Optional
+from typing import TYPE_CHECKING, List, Optional, cast
 
 from django.db import connections, transaction
 
@@ -7,6 +7,9 @@ from .config import PostgresPartitioningConfig
 from .constants import AUTO_PARTITIONED_COMMENT
 from .partition import PostgresPartition
 
+if TYPE_CHECKING:
+    from psqlextra.backend.schema import PostgresSchemaEditor
+
 
 @dataclass
 class PostgresModelPartitioningPlan:
@@ -28,7 +31,7 @@ class PostgresModelPartitioningPlan:
 
         Arguments:
             using:
-                Name of the database connection to use.
+                Optional name of the database connection to use.
         """
 
         connection = connections[using or "default"]
@@ -38,12 +41,15 @@ class PostgresModelPartitioningPlan:
                 for partition in self.creations:
                     partition.create(
                         self.config.model,
-                        schema_editor,
+                        cast("PostgresSchemaEditor", schema_editor),
                         comment=AUTO_PARTITIONED_COMMENT,
                     )
 
                 for partition in self.deletions:
-                    partition.delete(self.config.model, schema_editor)
+                    partition.delete(
+                        self.config.model,
+                        cast("PostgresSchemaEditor", schema_editor),
+                    )
 
     def print(self) -> None:
         """Prints this model plan to the terminal in a readable format."""
